@@ -11,7 +11,7 @@ method that must be called when you want to commit your changes to disk.
 
 """
 
-import hashlib, collections, struct
+import hashlib, collections, struct, logging
 
 from collections import MutableMapping
 from sqliteshelf import SQLiteShelf
@@ -587,6 +587,9 @@ l3yNr1m9i9inTKl+pWhOQC0rRg==
                 
                 if self.get_node_by_hash(self.get_node_hash(child)) != child:
                     raise Exception("Inconsistent insert")
+                    
+                if self.load_node(child).children is not None:
+                    raise Exception("Updated value on node with children")
                 
             elif child_key is not None:
                 # If the slot has a node with a different key hash,
@@ -615,6 +618,11 @@ l3yNr1m9i9inTKl+pWhOQC0rRg==
                 
                 if self.get_node_by_hash(self.get_node_hash(child)) != child:
                     raise Exception("Inconsistent insert")
+                    
+                if (self.get_node_key(child) is not None or 
+                    self.get_node_value(child) is not None):
+                    
+                    raise Exception("Node with children added still has value")
                 
             else:
                 # If the slot has a node with no key hash (i.e. it has
@@ -624,6 +632,11 @@ l3yNr1m9i9inTKl+pWhOQC0rRg==
                 
                 if self.get_node_by_hash(self.get_node_hash(child)) != child:
                     raise Exception("Inconsistent insert")
+                    
+                if (self.get_node_key(child) is not None or 
+                    self.get_node_value(child) is not None):
+                    
+                    raise Exception("Node with children added still has value")
             
         # Update our Merkle hash
         self.update_node_hash(node)
@@ -743,6 +756,10 @@ l3yNr1m9i9inTKl+pWhOQC0rRg==
                 # Kill the child
                 self.delete_node(child)
                 self.set_node_child(node, child_indices[0], None)
+                
+                if self.load_node(node).children is not None:
+                    raise Exception("Node left with children when value "
+                        "promoted")
             
         # Update our Merkle hash
         self.update_node_hash(node)
@@ -808,6 +825,56 @@ l3yNr1m9i9inTKl+pWhOQC0rRg==
             
             return self.recursive_find(child, key_hash, key, level + 1)
             
+    def audit(self):
+        """
+        Make sure the AuthenticatedDictionary is in a consistent state.
+        
+        """
+        
+        # Put some statistics
+        logging.debug("TRIE AUDIT: {} nodes on disk, {} updates, {} "
+            "deletes".format(len(self.store), len(self.updates),
+            len(self.deletes)))
+            
+        # What do we need to look at?
+        stack = []
+        stack.append("root")
+        
+        root_hash = self.get_hash()
+        
+        while len(stack) > 0:
+            # Pop a node to look at
+            node = stack.pop()
+            
+            if self.updates.has_key(node) and node in self.deletes:
+                logging.error("Node {} both updated and deleted".format(node))
+            
+            # Load the node's struct
+            node_struct = self.load_node(node)
+            
+            # Compute what its hash should be
+            expected_hash = hashlib.sha512(self.node_to_bytes(node)).digest()
+            
+            if expected_hash != node_struct.hash:
+                logging.error("Hash mismatch on node {}".format(node))
+            
+            if node_struct.children is not None:
+                if node_struct.key is not None or node_struct.value is not None:
+                    logging.error("Both value and children on node " 
+                        "{}".format(node))
+                for child in node_struct.children:
+                    if child is not None:
+                        # It actually has this child, so look at it.
+                        stack.append(child)
+            else:
+                if node_struct.key is None or node_struct.value is None:
+                    logging.eror("Neither value nor key on node {}".format(
+                        node))
+                
+                if self.find(node_struct.key) != node_struct.value:
+                    logging.info("Node {} could not be found by search.".format(
+                        node))
+            
     def get_node_children(self, node):
         """
         Given the pointer of a node, return a sequence of either child
@@ -844,6 +911,16 @@ l3yNr1m9i9inTKl+pWhOQC0rRg==
         # Update the node
         node_struct.children[index] = child
         
+        if child is None:
+            # We may have removed the last real child.
+            # Count up all the not-None children
+            actual_children = sum(1 for c in node_struct.children 
+                if c is not None)
+                    
+            if actual_children == 0:
+                # No real children remain. Get rid of the list of Nones
+                node_struct.children = None
+                
         # Save the node again
         self.store_node(node, node_struct)
         
