@@ -1191,6 +1191,9 @@ class Blockchain(object):
                         # We stepped back and got an incorrect state. This
                         # should never happen because these blocks have already
                         # been validated.
+                        
+                        self.state.audit()
+                        
                         raise Exception("Stepped back into wrong state after "
                             "block {}".format(pybc.bytes2string(position)))
                 
@@ -1745,6 +1748,11 @@ class Blockchain(object):
                 # We already know about this one. Don't bother trying to verify
                 # it. It won't verify twice, and will just produce console spam.
                return
+               
+            if len(self.transactions) >= 100:
+                # This is too many. TODO: make this configurable
+                logging.info("Rejecting transaction due to full queue")
+                return
             
             if not self.state_available:
                 # We can't trust our stored state, or the transaction_state that
