@@ -539,10 +539,13 @@ class CoinState(State):
             unused_output = (transaction_hash, i, amount, destination)
             
             if self.has_unused_output(unused_output):
-                loging.error("Trying to create duplicate output: {} {} {} "
+                logging.error("Trying to create duplicate output: {} {} {} "
                     "{}".format(util.bytes2string(unused_output[0]), 
                     unused_output[1], unused_output[2], 
                     util.bytes2string(unused_output[3])))
+                
+                self.audit()    
+                
                 # See if things just work out
             else:
                 self.add_unused_output(unused_output)
@@ -1064,10 +1067,23 @@ class CoinBlockchain(TransactionalBlockchain):
             reward_transaction = Transaction()
             reward_transaction.add_output(total_fee, destination)
             
-            to_include.append(reward_transaction)
+            # This may match exactly the reward transaction in the last block if
+            # we just made and solved that. What unused output does the
+            # generation transaction produce?
             
-            # Don't forget to use the reward transaction in the new State.
-            next_state.apply_transaction(reward_transaction)
+
+            if not self.verify_transaction(reward_transaction.to_bytes(), 
+                self.highest_block, next_state, advance=True):
+                    
+                # Our reward-taking transaction is invalid for some reason. We
+                # probably already generated a block this second or something.
+                
+                logging.info("Reward transaction would be invalid (maybe "
+                    "already used). Skipping block creation.")
+                
+                return None
+            
+            to_include.append(reward_transaction)
             
             # Make a block moving to the state we have after we apply all those
             # transactions, with the transaction packed into its payload
